﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using DataAccess;
using DomainClasses;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Data;

namespace IntegrationTests
{
  [TestClass]
  public class UnitTest1
  {
    public UnitTest1()
    {

      Database.SetInitializer(new MyInitializer());
    }
    [TestMethod]
    public void CanSetNavPropertyInNewEntityAndSaveInContext()
    {
      using (var context = new ScreencastContext())
      {
        var dataTopic = context.Topics.FirstOrDefault(t => t.Name.Contains("Data"));
        var originalTopicId = dataTopic.Id;
        context.Screencasts.Add(new Screencast
                                  {
                                    Title = "EF101",
                                    Description = "Entity Framework 101",
                                    Topic = dataTopic
                                  });
        context.SaveChanges();

      }

    }

    [TestMethod]
    public void SetNavPropertyInNewEntityAndSaveDisconnectedCreatesNewTopicAndId()
    {
      Screencast screencast;
      int originalTopicId;
      using (var context = new ScreencastContext())
      {
        var dataTopic = context.Topics.FirstOrDefault(t => t.Name.Contains("Data"));
        screencast = new Screencast { Description = "Entity Framework 102", Topic = dataTopic };
      }
      using (var context = new ScreencastContext())
      {
        originalTopicId = screencast.Topic.Id;
        context.Screencasts.Add(screencast);
        context.SaveChanges();
        Assert.IsTrue(screencast.TopicId != originalTopicId);
      }

    }
    [TestMethod]
    public void CanSetFkInNewEntityAndSaveDisconnected()
    {
      Screencast screencast;
      int originalTopicId;
      using (var context = new ScreencastContext())
      {
        var dataTopic = context.Topics.FirstOrDefault(t => t.Name.Contains("Data"));
        originalTopicId = dataTopic.Id;
        screencast = new Screencast
        {
          Description = "Entity Framework 102",
          TopicId = originalTopicId
        };
      }
      using (var context = new ScreencastContext())
      {
        context.Screencasts.Add(screencast);
        context.SaveChanges();
        Assert.IsTrue(screencast.TopicId == originalTopicId);
      }
    }

    [TestMethod]
    public void AuthorIdRemainsSameAfterAddAuthorsToScreencastAuthorsAndSaveConnected()
    {
      Screencast screencast;
      int origAuthorIdA;
      int origAuthorIdB;
      List<Author> twoAuthors;
      using (var context = new ScreencastContext())
      {
        twoAuthors = context.Authors.Take(2).ToList();
        origAuthorIdA = twoAuthors[0].Id;
        origAuthorIdB = twoAuthors[1].Id;
        var cast = new Screencast { Title = "EF 201", Description = "Entity Framework 201", TopicId = 1 };
        twoAuthors.ForEach(a => cast.Authors.Add(a));
        context.Screencasts.Add(cast);
        context.SaveChanges();
        Assert.AreEqual(origAuthorIdA, twoAuthors[0].Id);
        Assert.AreEqual(origAuthorIdB, twoAuthors[1].Id);
      }

    }
    [TestMethod]
    public void AuthorIdIsNewAfterAddAuthorsToNewScreencastAuthorsAndSaveDisconnected()
    {
      Screencast screencast;
      int origAuthorIdA;
      int origAuthorIdB;
      List<Author> twoAuthors;
      using (var context = new ScreencastContext())
      {
        twoAuthors = context.Authors.Take(2).ToList();
      }
      origAuthorIdA = twoAuthors[0].Id;
      origAuthorIdB = twoAuthors[1].Id;
      var cast = new Screencast { Title = "EF 201", Description = "Entity Framework 201", TopicId = 1 };
      twoAuthors.ForEach(a => cast.Authors.Add(a));
      using (var context = new ScreencastContext())
      {
        context.Screencasts.Add(cast);
        context.SaveChanges();
        Assert.AreNotEqual(origAuthorIdA, twoAuthors[0].Id);
        Assert.AreNotEqual(origAuthorIdB, twoAuthors[1].Id);
      }

    }
    [TestMethod]
    public void AuthorIdRemainsSameAfterAddAuthorsToNewScreencastAuthorsAndSaveDisconnectedWithStateFixup()
    {
      //scenario: app allows you to create screencasts from existing or new authors
      Screencast screencast;
      int origAuthorIdA;
      int origAuthorIdB;
      List<Author> twoAuthors;
      using (var context = new ScreencastContext())
      {
        twoAuthors = context.Authors.Take(2).ToList();
      }
      origAuthorIdA = twoAuthors[0].Id;
      origAuthorIdB = twoAuthors[1].Id;
      var cast = new Screencast { Title = "EF 201", Description = "Entity Framework 201", TopicId = 1 };
      twoAuthors.ForEach(a => cast.Authors.Add(a));
      using (var context = new ScreencastContext())
      {
        context.Screencasts.Add(cast);
        //explicitly fix state
        cast.Authors.Where(a => a.Id > 0).ToList().ForEach(a => context.Entry(a).State = EntityState.Unchanged);

        context.SaveChanges();
        Assert.AreEqual(origAuthorIdA, twoAuthors[0].Id);
        Assert.AreEqual(origAuthorIdB, twoAuthors[1].Id);
      }

    }
    [TestMethod]
    public void AuthorIdRemainsSameAfterAddAuthorsToNewScreencastAuthorsAndSaveDisconnectedWithAttachAndStateFixup()
    {
      //scenario: app only allows you to create screencasts from EXISTING authors!
      Screencast screencast;
      int origAuthorIdA;
      int origAuthorIdB;
      List<Author> twoAuthors;
      using (var context = new ScreencastContext())
      {
        twoAuthors = context.Authors.Take(2).ToList();
      }
      origAuthorIdA = twoAuthors[0].Id;
      origAuthorIdB = twoAuthors[1].Id;
      var cast = new Screencast { Title = "EF 201", Description = "Entity Framework 201", TopicId = 1 };
      twoAuthors.ForEach(a => cast.Authors.Add(a));
      using (var context = new ScreencastContext())
      {
        context.Screencasts.Attach(cast);
        //explicitly fix state
        context.Entry(cast).State = EntityState.Added;
        cast.Authors.ForEach(a => context.Entry(a).State = EntityState.Unchanged);

        context.SaveChanges();
        Assert.AreEqual(origAuthorIdA, twoAuthors[0].Id);
        Assert.AreEqual(origAuthorIdB, twoAuthors[1].Id);
      }

    }

    [TestMethod]
    public void MoveExistingSectionsToNewScreecastSetsFkToNewScreencastId()
    {
      var sc = new Screencast { Title = "EFLong", Description = "Really Long Course", TopicId = 3 };
      var section = new ScreenCastSection { Name = "Part 0" };
      sc.Sections.Add(new ScreenCastSection { Name = "Part 1" });
      sc.Sections.Add(new ScreenCastSection { Name = "Part 2" });
      sc.Sections.Add(new ScreenCastSection { Name = "Part 10" });

      using (var context = new ScreencastContext())
      {
        context.Screencasts.Add(sc);
        context.SaveChanges();
      }
      var newScreencast = new Screencast { Title = "EF Part 2", Description = "Second Part of Long Course", TopicId = 3 };
      var part10 = sc.Sections.FirstOrDefault(s => s.Name == "Part 10"); //note I'm looking in sc, not a context...and therefore not the database
      newScreencast.Sections.Add(part10);
      var originalSection10Id = part10.Id;
      using (var context = new ScreencastContext())
      {
        context.ScreenCastSections.Attach(part10);
        context.Screencasts.Add(newScreencast);
        context.SaveChanges();
      }
      Assert.AreEqual(part10.Id, originalSection10Id);
    }

  }
}
